#ifndef CNGGGPS_INDIVIDUAL
#define CNGGGPS_INDIVIDUAL

#include <string>
#include "Grammar.hpp"
#include "GrammarRules.hpp"
#include "Context.hpp"
#include "Result.hpp"

/**
 * \brief Individual
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * A program, which may be modified or executed by the GP system.  This object
 * also contains the fitness and number of hits of the stored individual as
 * determined the last time it was evaluated.
 */
class Individual
{
public:
	/**
	 * Empty constructor for creating Individual objects that are not
	 * initialized.
	 */
	Individual();

	/**
	 * Creates an individual object with metainformation concerning
	 * this individual.
	 *
	 * \param[in,out] pcontext	
	 * 	A reference to the context of execution	
	 * \param[in]	  bInitialize	
	 * 	If true, below are set to class variables and root node of 
	 * 	GStart is set.
	 * \param[in]	  iMinDepth	
	 * 	The minimum depth the symbol tree is allowed to be.
	 * \param[in] 	  iMaxDepth
	 *	The maximum depth the symbol tree is allowed to be
	 */
	Individual(Context *pcontext, bool bInitialize=true, int iMinDepth=0, int iMaxDepth=0);

	/**
	 * Selects a random grammar symbol from the individual.  The symbol
	 * returned is guaranteed not to be a leaf node.
	 * 
	 * \return 
	 * 	A pointer to a Grammar object in the individual's
	 * 	symbol tree
	 */
	Grammar* getRandomNode() const;

	/**
	 * Selects a random grammar symbol from the individual which
	 * matches the symbol type defined by the passed in grammar ID.
	 *
	 * \param[in]	strGrammarID
	 * 	A string to be matched against the grammar ID's of the symbols
	 * 	in the symbol tree
	 * \return
	 * 	A pointer to a Grammar object in the individual's
	 * 	symbol tree
	 */
	Grammar* getRandomNodeMatching(std::string strGrammarID) const;
	
	/**
	 * \return
	 * 	The fitness of this individual as determined by the Evaluator
	 * \sa
	 * 	Evaluator::evaluate
	 */
	double getFitness() const;

	/**
	 * \param[in]	dFitness
	 * 	A fitness value to be associated with this individual
	 */
	void setFitness(double dFitness);

	/**
	 * \return
	 * 	The number of hits this individual obtained as determined by
	 * 	the Evaluator
	 * \sa
	 * 	Evaluator::evaluate
	 */
	int getHits() const;

	/**
	 * \param[in]	iHits
	 *	A hit count to be associated with this individual
	 */
	void setHits(int iHits);

	/**
	 * \return
	 * 	The minimum depth which this individual's symbol tree is allowed
	 * 	have.
	 */
	int getMinDepth() const;

	/**
	 * \return
	 * 	The maximum depth which this individual's symbol tree is allowed
	 * 	to have.
	 */
	int getMaxDepth() const;

	/**
	 * Forms a LISP expression representing the individual's phenotype
	 *
	 * \return
	 * 	A string representing this individual
	 */
	std::string toString() const;

	/**
	 * Creates a subtree for the given node.  
	 * \warning 
	 * 	The node should be part of this individual. While this is not a
	 * 	formal requirement in this version it may be changed in the
	 * 	future.
	 * \param[in,out] pgramNode
	 * 	A pointer to the node for which the subtree will be built.
	 * \param[in]	  iTries
	 * 	The number of attempts at building a subtree within the
	 * 	individual's contraints before the process is abandoned.
	 * 	The value 0 is used when infinite tries may be made.
	 */
	bool buildTree(Grammar* pgramNode, int iTries);
	
	/**
	 * Forms a vector of all the nodes in the symbol tree for fast access.
	 * \note
	 * 	This method should be called after any direct modifcation
	 * 	to the symbol tree.
	 */
	void formNodeList();

	/**
	 * Runs the program defined by the symbol tree.
	 *
	 * \param[in,out]	context
	 * 	The argument values the program uses.  These may be modified
	 * 	by the program and should be reset between executions if
	 * 	there are any symbols included in the language which do.
	 * \return
	 * 	The program output
	 */
	Result* execute(Context &context);

	/**
	 * Performs a deep copy, creating another Individual identical to
	 * this.
	 *
	 * \return
	 * 	A new Individual with indentical structure but in its own
	 * 	memory
	 */
	Individual* clone() const;	
	
	~Individual();
private:
	/**
	 * Recursive method used in forming the node list.  Never called
	 * directly outside of itself and formNodeList
	 *
	 * \param[in,out]	pgramNode
	 * 	A node to be added to the list along with it's subtree
	 * \sa
	 * 	formNodeList
	 */
	void addSubtreeToList(Grammar* pgramNode);

	/**
	 * A recursive method used by buildTree to create a symbol tree.
	 * This method picks a rule from GrammarRules, adds the symbols
	 * from that rule to the current node, and calls itself on each
	 * of those symbols
	 *
	 * \param[in,out]	pgramNode
	 * 	The symbol to be built upon
	 * \param[in]		iDepth
	 * 	The depth of the current node in the symbol tree
	 * \return
	 * 	true if the created tree is within the constraints of the
	 * 	individual, false otherwise.
	 */
	bool addNodeToTree(Grammar* pgramNode, int iDepth);

	/**
	 * The root node of the symbol tree for this individual.  Expected to
	 * be of type GStart
	 */
	Grammar* pgramRoot;

	///Hit count associated with this individual
	int iHits;

	///Fitness score of this individual
	double dFitness;
	
	///The depth of the deepest node in this individual
	int iDepth;

	///The minimum depth allowed for a valid symbol tree
	int iMinDepth;

	///The maximum depth allowed for a valid symbol tree
	int iMaxDepth;

	///A vector of all the nodes in the symbol tree for fast access
	std::vector<Grammar*> vpgramNodeList;

	///A context, storing argument values.  May be deleted.
	Context* pcontext;
};
#endif

