#ifndef CNGGGPS_LOG
#define CNGGGPS_LOG

#include <vector>
#include <string>
#include "Individual.hpp"
#include "SLogEntry.hpp"

/**
 * \brief Logging system.
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 *
 * Formats information from the genetic programming system, such as fitness
 * values per generation and print outs of the best solutions to files where
 * they can be analysed later.
 */
class Log
{
public:
	/**
	 * Sets up a new logging system.  Most of the parameters are used for
	 * formatting the data.
	 *
	 * \param[in] strName
	 * 	The base name, extended to create the various files.
	 * \param[in] iRuns
	 * 	The number of runs or trials the GP will cycle through
	 * \param[in] iGenerations
	 * 	The number of cycles per run
	 */
	Log(std::string strName, int iRuns = 5, int iGenerations = 100);

	/**
	 * Logs the parameters to a file.
	 *
	 * \param[in] iPopulationSize
	 * 	Number of individuals in a population.
	 * \param[in] dCrossoverRate
	 * 	Proportion of population to be crossed.
	 * \param[in] dMutationRate
	 * 	Proportion of population to be mutated.
	 * \param[in] iTries
	 * 	Number of attempts at crossover or mutation made before it is
	 * 	abandoned.
	 * \param[in] iMinDepth
	 * 	Minimum distance between root and furthest leaf to be considered
	 * 	a valid tree.
	 * \param[in] iMaxDepth
	 * 	Maximum distance between root and furthest leaf to be considered
	 * 	a valid tree.
	 * \param[in] iTournamentSize
	 * 	Number of elements randomly selected from the population of
	 * 	which the one with the highest fitness is selected.
	 */
	void logParameters(int iPopulationSize, double dCrossoverRate, double dMutationRate, int iTries, int iMinDepth, int iMaxDepth, int iTournamentSize);

	/**
	 * Add fitness information for a particular generation of a particular
	 * run.
	 *
	 * \param[in] iRun
	 * 	The run for which the information pertains.
	 * \param[in] iGeneration
	 * 	The generation for which the information pertains.
	 * \param[in] dBestFitness
	 * 	The best fitness of any individual in this run and generation.
	 * \param[in] dAvgFitness
	 * 	The average fitness of this run and generation.
	 * \param[in] dBestHits
	 * 	The number of hits obtained by the individual with the best
	 * 	fitness of this run and generation.
	 * \param[in] dAvgHits
	 * 	The average hits of this run and generation.
	 */
	void addEntry(int iRun, int iGeneration, double dBestFitness, double dAvgFitness, int iBestHits, double dAvgHits);

	/**
	 * Write the best individuals to file.  This list must be generated
	 * elsewhere and passed in.
	 *
	 * \param[in] vpindBest
	 * 	A list of the best individual from each run in order.
	 */
	void logBest(std::vector<Individual*> vpindBest);

	~Log();
private:
	std::vector<SLogEntry> vle;
	std::vector<std::vector<SLogEntry> > vvle;
	
	/**
	 * The base name of the experiment.  Log files will consist of this
	 * name plus an extention.
	 */
	std::string strName;

	/**
	 * The number of runs the GP system will go through for this experiment.
	 */
	int iRuns;

	/**
	 * The number of generations or cycles per run.
	 */
	int iGenerations;

	/**
	 * Records the statistics gathered to files.
	 */
	void dumpToFile();
};

#endif
