#ifndef CNGGGPS_POPULATION
#define CNGGGPS_POPULATION

#include <vector>
#include "Individual.hpp"
#include "Evaluator.hpp"

/**
 * \brief A container for holding a collection of individual programs
 * \author Stephen E. Baker
 * \date 2006-2007
 * \version 1.0
 */
class Population
{
public:
	/**
	 * Create a new population object.  This merely sets the population
	 * up, and does not populate it with individuals.
	 * 
	 * \param[in,out] pcontext
	 * 	A pointer to the context for the GP.
	 * \param[in] peval
	 * 	A pointer to the evaluator object.
	 * \param[in] iPopulationSize
	 * 	The number of individuals to allocate space for in the
	 * 	population.
	 * \param[in] iTournamentSize
	 * 	The size of the pool of random individuals from which the best
	 * 	is chosen in the selection method.
	 * \param[in] iMinDepth
	 * 	The minimum depth an individual in this population's symbol tree 	 * 	must be to be considered valid.
	 * \param[in] iMaxDepth
	 * 	The maximum depth an individual in this population's symbol tree
	 * 	must be to be considered valid.
	 * \param[in] iTries
	 * 	The number of attempts a particular mutation or crossover will
	 * 	try before it will fail.
	 */
	Population(Context* pcontext, Evaluator* peval, int iPopulationSize = 100, int iTournamentSize=3, int iMinDepth=0, int iMaxDepth=30, int iTries = 5);

	/**
	 * Constructs enough random individuals to fill the population.
	 */
	void generateInitialPopulation();

	/**
	 * Returns a copy of the individual in the population with the highest 
	 * fitness value.  A pointer to this individual is kept when the
	 * population is evaluated so this can be done in constant time.
	 *
	 * \sa
	 * 	evaluate
	 */
	Individual* getBestIndividual() const;

	/**
	 * Performs tournament selection to choose an individual from the
	 * population.  Specifically this picks a number of individuals from
	 * the population at random and returns the one with the highest
	 * fitness.
	 * 
	 * \return
	 * 	A copy of the selected individual
	 * \sa
	 * 	iTournamentSize
	 */
	Individual* selectIndividual() const;

	/**
	 * Adds the given individual to the population.
	 *
	 * \param[in] ind
	 * 	The individual to be added to the population
	 */
	void addIndividual(Individual* ind);

	/**
	 * Returns the highest fitness value of any individual in the
	 * population.
	 */
	double getBestFitness() const;
	
	/**
	 * Returns the number of hits in the individual with the best fitness
	 * \note 
	 * 	This may not be the number of hits from the individual 
	 * 	with the most hits, but is consistent with other GP systems
	 * 	eg. OpenBEAGLE
	 */
	int getBestHits() const;

	/**
	 * Returns the mean fitness value of the individuals in the population.
	 */
	double getAverageFitness() const;

	/**
	 * Returns the mean hit value of the individuals in the population.
	 */
	double getAverageHits() const;

	/**
	 * Performs mutations on a given proportion of the population.  The
	 * actual individuals mutated are chosen at random.
	 *
	 * \param[in] dMutationRate
	 * 	The proportion to be mutated
	 * \sa
	 * 	Mutate
	 */
	void mutate(double dMutationRate);

	/**
	 * Performs crossover on a given proportion of the population.  The
	 * individuals crossed over are chosen at random, individuals are
	 * crossed with neighboring individuals in the population list.
	 * Since the order of the list is made random with the selection
	 * process in each generation this does not mean that the same
	 * two individuals are always crossed.
	 *
	 * \param[in] dCrossoverRate
	 * 	The proportion of individuals to be crossed
	 * \sa
	 * 	Crossover
	 */
	void crossover(double dCrossoverRate);

	/**
	 * Evaluates the entire population, updating the best individual pointer
	 * and averages to reflect the new values.  This should be called
	 * between any modification to the population and any access to these
	 * statistics.
	 *
	 * \sa
	 * 	Evaluator
	 */
	void evaluate();
	
	~Population();

private:
	/**
	 * All the individuals in the population.
	 */
	std::vector<Individual*>* pvpindPopulation;
	
	/**
	 * A pointer to the individual with the highest fitness value.
	 */
	Individual* pindBest;

	/**
	 * The number of individuals chosen at random for the selection pool
	 * from which the one with the highest fitness is chosen.
	 */
	int iTournamentSize;

	/**
	 * The maximum number of individuals which can be stored in the
	 * population.  Also the number of individuals created in
	 * generateInitialPopulation.
	 */
	int iPopulationSize;

	/**
	 * The minimum depth an individual's symbol tree may be to be considered
	 * valid.
	 */
	int iMinDepth;

	/**
	 * The maximum depth an individual's symbol tree may be to be considered
	 * valid.
	 */
	int iMaxDepth;

	/**
	 * The number of tries mutation or crossover will attempt before giving
	 * up.
	 */
	int iTries;

	/**
	 * The mean fitness value of individuals in the population.
	 */
	double dAvgFitness;

	/**
	 * The mean number of hits of individuals in the population.
	 */
	double dAvgHits;

	/**
	 * A pointer to the evaluator object for determininig an individual's
	 * fitness.
	 */
	Evaluator* peval;

	/**
	 * A pointer to the context object passed to individuals when they
	 * are evaluated.
	 */
	Context* pcontext;
};

#endif

