#ifndef CNGGGPS_RESULT
#define CNGGGPS_RESULT

#include<string>

/**
 * \brief A generic value storage class.  
 * \author Stephen E. Baker
 * \date 2006-2007
 *
 * The original idea of this class was to create a type that could be cast
 * to or from any other type.  The result fell somewhat short of that dream
 * and is a type which can be cast from any type back to the same type, and
 * in some cases a few others.
 * \note 
 * The resulting class could be replaced by one using templates (otherwise
 * called generics) with some tradeoffs.
 */
class Result
{
public:
	/**
	 * Creates a new empty result object.
	 */
	Result();

	/**
	 * Creates a result object storing an arbitrary block of memory.
	 *
	 * \param[in] vpValue
	 * 	A pointer to the block of memory to be stored.
	 * \param[in] size
	 * 	The size in bytes of the memory to be stored.
	 * \param[in] strType
	 * 	A string specifying the type of value stored in the memory
	 * 	being copied.
	 */
	Result(void* vpValue, size_t size, std::string strType = "void*");

	/**
	 * Creates a result object storing an int.
	 *
	 * \param[in] iValue
	 * 	The int to be stored
	 */
	Result(int iValue);

	/**
	 * Creates a result object storing a double.
	 *
	 * \param[in] dValue
	 * 	The double to be stored
	 */
	Result(double dValue);

	/**
	 * Creates a result object storing a bool.
	 *
	 * \param[in] bValue
	 * 	The bool to be stored
	 */
	Result(bool bValue);

	/**
	 * Creates a result object storing the contents of another Result
	 * object.
	 *
	 * \param[in] pres
	 * 	The result to be copied.
	 */
	Result(Result* pres);

	/**
	 * Set this result to store an arbitrary block of memory.
	 *
	 * \param[in] vpValue
	 * 	A pointer to the block of memory to be stored.
	 * \param[in] size
	 * 	The size in bytes of the memory to be stored.
	 * \param[in] strType
	 * 	A string specifying the type of value stored in the memory
	 * 	being copied.
	 */
	void setValue(void* vpValue, size_t size, std::string strType);

	/**
	 * Sets this result to store a double.
	 *
	 * \param[in] dValue
	 * 	The double to be stored.
	 */
	void setValue(double dValue);

	/**
	 * Sets this result to store an int.
	 *
	 * \param[in] iValue
	 * 	The int to be stored.
	 */
	void setValue(int iValue);

	/**
	 * Sets this result to store a bool.
	 *
	 * \param[in] bValue
	 * 	The bool to be stored.
	 */
	void setValue(bool bValue);

	/**
	 * Sets this result to store the contents of another result.
	 *
	 * \param[in] pres
	 * 	The result to be copied.
	 */
	void setValue(Result* pres);

	/**
	 * Returns a pointer to the value stored in this result.
	 * \note
	 * 	When the value is copied into the result it loses it's type
	 * 	information and must be cast back.
	 */
	void* getValue() const;

	/**
	 * Returns the value stored in this result as a double if the
	 * conversion is known.
	 */
	double toDouble() const;

	/**
	 * Returns the value stored in this result as an int if the conversion
	 * is known.
	 */
	int toInt() const;

	/**
	 * Returns the value stored in this result as a bool if the conversion
	 * is known.
	 */
	bool toBool() const;

	/**
	 * Returns the size of the data stored in the result.
	 */
	size_t getSize() const;

	~Result();
private:
	/**
	 * A pointer to the memory block where the value is stored.
	 */
	void* vpValue;

	/**
	 * The size of the block of memory being stored.
	 */
	size_t size;
	
	/**
	 * The type of value being stored in the result.
	 */
	std::string strType;
};

#endif

